package Module::Name;
use strict;
use warnings;

our $VERSION = '0.01';


=head1 NAME

Module::Name - One line name

=head1 SYNOPSIS

Short examples for the inpatient

=head1 DESCRIPTION

Detailed description of the script

=cut

1;
